//  ChatClient.java
//
//  Last modified 1/30/2000 by Alan Frindell
//
//  Chat Client starter application.
//
//  You will have to verify client handle and generate the keys 
//  (as part of the connect method)
//  You will be required to actually encrypt the messages for
//  programming project 1.

package Chat;

//  RMI
import java.rmi.*;

//  AWT/Swing
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

//  Java
import java.io.*;
import java.math.BigInteger;

//  Crypto
import java.security.*;
import java.security.spec.*;
import java.security.interfaces.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import javax.crypto.interfaces.*;

public class ChatClient  {

    public static final int SUCCESS = 0;
    public static final int CONNECTION_REFUSED = 1;
    public static final int BAD_HOST = 2;
    public static final int ERROR = 3;

    String _handle;//can be used as the username to identify with the server
    Integer _clientID;

    ChatServer _server;
    ChatConsumer _consumer;
    ChatLoginPanel _loginPanel;
    ChatRoomPanel _chatPanel;

    CardLayout _layout;
    JFrame _appFrame;
  
  
    //  ChatClient constructor
    //
    //  empty, as you can see.

    public ChatClient() {
	_handle = null;
	_clientID = new Integer(-1);
	_server = null;
	_consumer = null;

	try {
	    initComponents();
	} catch (Exception e) {
	    System.out.println("ChatClient error: " + e.getMessage());
	    e.printStackTrace();
	}

	_layout.show(_appFrame.getContentPane(), "Login");

    }


    public void run() {
	_appFrame.pack();
	_appFrame.setVisible(true);

    }

    //  main
    //
    //  Construct the app inside a frame, in the center of the screen

    public static void main(String[] args) {
	Security.addProvider(new com.sun.crypto.provider.SunJCE());

	ChatClient app = new ChatClient();
	
	app.run();
    }


    //  initComponents
    //
    //  Component initialization
    
    private void initComponents() throws Exception {
	_appFrame = new JFrame("CS255 Chat");
	_layout = new CardLayout();
	_appFrame.getContentPane().setLayout(_layout);
	_loginPanel = new ChatLoginPanel(this);
	_chatPanel = new ChatRoomPanel(this);
	_appFrame.getContentPane().add(_loginPanel, "Login");
	_appFrame.getContentPane().add(_chatPanel, "ChatRoom");
	_appFrame.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    quit();
		}
	    });
    }
  

    //  quit
    //
    //  Called when the application is about to quit.

    public void quit() {
	try {
	    if (_server != null) {
		_server.unregister(_clientID);
	    }
	} catch (Exception err) {
	    System.out.println("ChatClient error: " + err.getMessage());
	    err.printStackTrace();
	}
	System.exit(0);
    }
    


    //  connect
    //
    //  Called from the login panel when the user clicks the "connect"
    //  button.  Right now, just lookup the server object, register and
    //  off we go.  

    //  You will need to modify this function to perform a key
    //  generation.  
    //  Use the handle to verify with the server.  On successfull verification
    //  the password will be used to generated the necessary keys.
    
    public int connect(String handle, char []password, 
		       String host, int port) {
	try {
	    
	    _handle = handle;
	    _server = (ChatServer)Naming.lookup("//" + host + ":" + 
						port + "/ChatServer");
	    
	    _consumer = new ChatConsumerImpl(this, _chatPanel.getOutputArea());

	    _clientID = _server.getClientID();
	    _server.register(_consumer, _clientID);
	    
	    _layout.show(_appFrame.getContentPane(), "ChatRoom");

	    //use the handle to identify yourself
	    //if verifyHandle succeeds, then generate the keys.

	    return SUCCESS;
	} catch (AccessControlException accE) {
	    return BAD_HOST;
	} catch (RemoteException remoteE) {
	    return CONNECTION_REFUSED;
	} catch (Exception e) {
	    System.out.println("ChatClient err: " + e.getMessage());
	    e.printStackTrace();
	}
	
	return ERROR;
    }
    
    //  sendMessage
    //
    //  Called from the ChatPanel when the user types a carrige return.  You
    //  will need to modify this.
    
    public void sendMessage(String msg) {
	try {
	    String str = _handle + "> " + msg;

	    ChatMessage sealedMsg = encrypt(str, true);
	    _server.postMessage(sealedMsg, _clientID);

	} catch (Exception e) {
	    System.out.println("ChatClient err: " + e.getMessage());
	    e.printStackTrace();
	}
    }

    ChatMessage encrypt(String msg, boolean cbc) {
	try {

	// No encryption is done now.

        ChatMessage sealedMsg = new ChatMessage((Serializable)msg);
	
	    return sealedMsg;
	} catch (Exception e) {
	    System.out.println("ChatClient err: " + e.getMessage());
	    e.printStackTrace();
	}

	return null;
    }

    String decrypt(ChatMessage sealedMsg) {

	// No decryption is done now.

	try {
	    
	    String msg = (String)sealedMsg.getObject();

	    return msg;

	} catch (Exception e) {

	    System.out.println("ChatClient err: " + e.getMessage());
	    e.printStackTrace();
	}

	return null;
    }
}

