//  ChatConsumerImpl.java
//
//  Last modified 1/30/2000 by Alan Frindell
//
//  Implementation of the ChatConsumer callback object
//
//  This class is created by the ChatClient and sent to the ChatServer 
//  to receive all the messages posted to the chat room.
//
//  You will need to modify this code to implement the security features
//  of the chat system.

package Chat;

//  RMI
import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;

//  Swing
import javax.swing.JTextArea;

//  Crypto
import java.security.*;
import java.security.spec.*;
import java.security.interfaces.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import javax.crypto.interfaces.*;


public class ChatConsumerImpl extends UnicastRemoteObject
                              implements ChatConsumer {

    ChatClient _client;

    JTextArea _outputArea;
    
    //  ChatConsumerImpl constructor
    //
    //  Takes the text area where the messages should be displayed.
    
    public ChatConsumerImpl(ChatClient client, JTextArea outputArea) 
	throws RemoteException {
	
	_client = client;
	_outputArea = outputArea;
    }
    
    
    //  consumeMessage
    //
    //  Called by the server when a message has been posted to the chat
    //  room.  Append the message to the output area.
    
    public void consumeMessage(ChatMessage sealedMsg) throws RemoteException {
	//The server would send an encrypted message
	//Decrypt it before it goes to the output area

	String msg = (String)sealedMsg.getObject();

	if (msg != null) {
	    _outputArea.append(msg);
	}
    }
}
